#ifndef POAC_UTIL_ARGPARSE_HPP
#define POAC_UTIL_ARGPARSE_HPP

#include <type_traits>
#include <algorithm>
#include <utility>
#include <array>
#include <tuple>
#include <optional>

#include "./types.hpp"


namespace poac::util::argparse {
    template <class SinglePassRange, class... T>
    bool use(SinglePassRange& rng, T... args)
    {
        const auto first = std::begin(rng);
        const auto last = std::end(rng);
        return ((std::find(first, last, args) != last) || ...);
    }

    template <class SinglePassRange, class... T>
    bool use_rm(SinglePassRange& rng, T... args)
    {
        const auto first = std::begin(rng);
        const auto last = std::end(rng);
        bool found = false;
        for (const auto& a : types::tuple_to_array(std::tuple<T...>{ args... })) {
            if (const auto itr = std::find(first, last, a); itr != last) {
                rng.erase(itr);
                found = true;
            }
        }
        return found;
    }

    // -o filename -> return filename
    template <class SinglePassRange, class T>
    std::optional<std::string>
    use_get(SinglePassRange& rng, T arg) {
        const auto first = std::begin(rng);
        const auto last = std::end(rng);
        if (const auto result = std::find(first, last, arg); result != last) {
            return *(result + 1);
        }
        else {
            return std::nullopt;
        }
    }
    // -o filename -> return filename
    template <class SinglePassRange, class T>
    std::optional<std::string>
    use_get(SinglePassRange& rng, T arg1, T arg2) {
        const auto first = std::begin(rng);
        const auto last = std::end(rng);
        if (const auto result1 = std::find(first, last, arg1); result1 != last) {
            return *(result1 + 1);
        }
        else if (const auto result2 = std::find(first, last, arg2); result2 != last) {
            return *(result2 + 1);
        }
        else {
            return std::nullopt;
        }
    }
} // end namespace
#endif // !POAC_UTIL_ARGPARSE_HPP
